Index: asterisk-team-group-issue8824/channels/chan_dahdi.c
===================================================================
--- asterisk-team-group-issue8824.orig/channels/chan_dahdi.c	2008-12-11 14:37:15.000000000 +0200
+++ asterisk-team-group-issue8824/channels/chan_dahdi.c	2008-12-11 14:42:38.000000000 +0200
@@ -6065,6 +6065,30 @@
 		case AST_CONTROL_SRCUPDATE:
 			res = 0;
 			break;
+		case AST_CONTROL_CONNECTED_LINE:
+#ifdef PRI_CONNECTED_LINE
+			if (p->pri) {
+				struct ast_party_connected_line connected;
+
+				ast_party_connected_line_init(&connected);
+				if (!ast_parse_connected_line_data((unsigned char *) data, datalen, &connected)) {
+					ast_debug(1, "Called name changed to %s on channel %s\n",
+						  connected.id.name, chan->name);
+
+					if (!pri_grab(p, p->pri)) {
+						pri_call_set_connected_line(
+							p->call,
+							connected.id.name,
+							connected.id.number_presentation);
+						pri_rel(p->pri);
+					} else
+						ast_log(LOG_WARNING, "Unable to grab PRI on span %d\n", p->span);
+					res = 0;
+				}
+				ast_party_connected_line_free(&connected);
+			}
+#endif
+			break;
 		case -1:
 			res = tone_zone_play_tone(p->subs[idx].dfd, -1);
 			break;
@@ -11214,6 +11238,8 @@
 						pri->pvts[chanpos]->call = NULL;
 						pri->pvts[chanpos]->exten[0] = '\0';
 					}
+					if (pri->facilityenable)
+						pri_facility_enable(pri->pri);
 					if (crv)
 						ast_mutex_unlock(&crv->lock);
 					ast_mutex_unlock(&pri->pvts[chanpos]->lock);
@@ -11265,6 +11291,18 @@
 						}
 #endif
 
+#ifdef PRI_CONNECTED_LINE
+						if (!ast_strlen_zero(e->ringing.connectedname)) {
+							struct ast_party_connected_line connected;
+
+							ast_party_connected_line_init(&connected);
+							connected.id.name = (char *) e->ringing.connectedname;
+							connected.id.number_presentation = e->ringing.connectedpres;
+							connected.source = AST_CONNECTED_LINE_UPDATE_SOURCE_ANSWER;
+							ast_queue_connected_line_update(pri->pvts[chanpos]->owner, &connected);
+						}
+#endif
+
 						ast_mutex_unlock(&pri->pvts[chanpos]->lock);
 					}
 				}
@@ -11422,6 +11460,18 @@
 						}
 #endif
 
+#ifdef PRI_CONNECTED_LINE
+						if (!ast_strlen_zero(e->answer.connectedname)) {
+							struct ast_party_connected_line connected;
+
+							ast_party_connected_line_init(&connected);
+							connected.id.name = (char *) e->answer.connectedname;
+							connected.id.number_presentation = e->answer.connectedpres;
+							connected.source = AST_CONNECTED_LINE_UPDATE_SOURCE_ANSWER;
+							ast_queue_connected_line_update(pri->pvts[chanpos]->owner, &connected);
+						}
+#endif
+
 						ast_mutex_unlock(&pri->pvts[chanpos]->lock);
 					}
 				}
@@ -11488,6 +11538,18 @@
 						}
 #endif
 
+#ifdef PRI_CONNECTED_LINE
+						if (pri->pvts[chanpos]->owner && !ast_strlen_zero(e->hangup.connectedname)) {
+							struct ast_party_connected_line connected;
+
+							ast_party_connected_line_init(&connected);
+							connected.id.name = (char *) e->hangup.connectedname;
+							connected.id.number_presentation = e->hangup.connectedpres;
+							connected.source = AST_CONNECTED_LINE_UPDATE_SOURCE_ANSWER;
+							ast_queue_connected_line_update(pri->pvts[chanpos]->owner, &connected);
+						}
+#endif
+
 						ast_mutex_unlock(&pri->pvts[chanpos]->lock);
 					} else {
 						ast_log(LOG_WARNING, "Hangup on bad channel %d/%d on span %d\n", 
